# subscribe_api.py
import fire
from pybraries.subscription_helpers import sub_api
from typing import Dict, List, Optional, Any


class Subscribe(object):
    """Class for libraries.io API for changing user's libraries.io subscriptions"""

    def __init__(self):
        pass

    def list_subscribed(self) -> Any:
        """
        Return a list of packages a user is subscribed to for release notifications.

        Returns:
            Dict with info for each package subscribed to at libraries.io.
        """
        return sub_api("list_subscribed")

    def subscribe(self, manager: str, package: str) -> str:
        """
        Subscribe to receive notifications about new releases of a project.

        Not working yet, but hope to include - possible bug at libraries.io:
        include_prerelease: default = True. Include prerelease notifications.

        Args:
            manager: package manager name (e.g. PyPI).
            package: package name.
        Returns:
            Subscription confirmation message.
        """
        return sub_api("subscribe", manager, package)

    def check_subscribed(self, manager: str, package: str) -> bool:
        """
        Check if a user is subscribed to notifications for new project releases.

        Args:
            manager: package manager name (e.g. PyPI).
            package: package name.
        Returns:
            True if subscribed to the package indicated, else False.
        """
        return sub_api("check_subscribed", manager, package)

    def update_subscribe(
        self, manager: str, package: str, include_prerelease: bool
    ) -> str:
        """
        NOT IMPLEMENTED due to possible bug in libraries.io
        Update the options for a subscription.

        Args:
            manager: package manager name (e.g. PyPI).
            package: package name.
            include_prerelease (bool): default = True. Include prerelease notifications.

        Returns:
            Update confirmation message.
        """
        return sub_api("update_subscribe", manager, package, include_prerelease)

    def unsubscribe(self, manager: str, package: str) -> str:
        """
        Stop receiving release notifications from a project.

        Args:
            manager: package manager name (e.g. PyPI).
            package: package name.

        Returns:
            Message confirming delete or deletion unnecessary.
        """

        return sub_api("delete_subscribe", manager, package)


# From the command line you can call any public function by name with arguments
if __name__ == "__main__":
    fire.Fire(Subscribe)

    subs = Subscribe()

    # sub = subs.list_subscribed()
    # print(sub)

    x = subs.subscribe(manager="pypi", package="pandas")
    print(x)

    # a = subs.unsubscribe(manager="pypi", package="pandas")
    # print(a)

    # y = subs.check_subscribed("pypi", "numpy")
    # print(y)

    # z = subs.update_subscribe(
    #    manager="pypi", package="plotly", include_prerelease="False"
    # )
    # print(z)



# make_request.pyß
from requests.exceptions import HTTPError
from pybraries.helpers import sess, clear_params
from typing import Dict, List, Optional, Any


def make_request(url: str, kind: str) -> Any:
    """ call api server 

        Args:
            url (str): base url to call
            kind (str): get, post, put, or delete
        Returns:
            response from libraries.io
    
    """

    try:
        if kind == "get":
            r = sess.get(url)
            r.raise_for_status()
            r_val = r.json()
        if kind == "post":
            r = sess.post(url, params={"include_prerelease": "False"})

            print(r.request.url)
            print("")
            print(r.headers)
            r.raise_for_status()
            r_val = "successfully subscribed"
            x = r.json()
            print(x["include_prerelease"])
        if kind == "put":

            r = sess.put(url)
            print(r)
            print("body:")
            print(r.request.body)
            print(r.request.url)
            x = r.json()
            print(x["include_prerelease"])

            r.raise_for_status()
            r_val = "successfully updated"
        if kind == "delete":
            r = sess.delete(url)
            r.raise_for_status()
            r_val = "successfully unsubscribed"

        clear_params()
        return r_val

    except HTTPError as http_err:
        clear_params()
        print(f"HTTP error occurred: {http_err}")
    except Exception as err:
        clear_params()
        print(f"Other error occurred: {err}")



import os
import requests
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter


class APIKeyMissingError(Exception):
    """Custom error for API Key missing"""

    pass


LIBRARIES_API_KEY = os.environ.get("LIBRARIES_API_KEY", None)

if LIBRARIES_API_KEY is None:
    raise APIKeyMissingError(
        "All methods require an API key. "
        "See https://libraries.io to get your free key. "
        "Then set the key to the environment variable: LIBRARIES_API_KEY"
    )

# session retry settings
retries = Retry(total=3, backoff_factor=0.2, status_forcelist=[500, 502, 503, 504])

# session object common properties
sess = requests.Session()
# sess.params = {"include_prerelease": False}
sess.params["api_key"] = LIBRARIES_API_KEY
# sess.params["include_prerelease"] = 0
sess.mount("https://", HTTPAdapter(max_retries=retries))


def clear_params() -> None:
    sess.params.clear()
    sess.params["api_key"] = LIBRARIES_API_KEY



# search.py
import fire
from pybraries.search_helpers import search_api
from pybraries.set_pages import set_pages
from typing import Dict, List, Optional, Any


class Search(object):
    """Class for wrapping the libraries.io API for 
    platform, project, repo, and user GET actions"""

    def __init__(self):
        pass

    def platforms(self) -> Any:
        """
        Return a list of supported package managers.

        Returns:
            List of dicts of platforms with platform info from libraries.io.
        """

        return search_api("platforms")

    def project(self, manager: str, package: str) -> Any:
        """
        Return information about a package and its versions from a platform (e.g. PyPI).

        Args:
            manager: package manager (e.g. "pypi").
            package: package name.
        Returns:
            Dict of information about the project from libraries.io.
        """

        return search_api("pproject", manager, package)

    def project_dependencies(self, manager: str, package: str) -> Any:
        """
        Get a list of dependencies for a version of a project.

        Returns latest version info.

        Args:
            manager: package manager (e.g. "pypi").
            package: package name.
        Returns:
            Dict of dependencies for a version of a project from libraries.io.
        """

        return search_api("pproject_dependencies", manager, package)

    def project_dependents(self, manager: str, package: str) -> Any:
        """
        Get packages that have at least one version that depends on a given project.

        Args:
            manager: package manager (e.g. "pypi").
            package: package name
            version: package version
        Returns:
            List of dicts package dependents from libraries.io.
        """

        return search_api("pproject_dependendents", manager, package)

    def project_dependent_repositories(self, manager: str, package: str) -> Any:
        """
        Get repositories that depend on a given project.

        Args:
            manager: package manager (e.g. "pypi")
            package: package name
        Returns:
            List of dicts of dependent repositories from libraries.io.
        """

        return search_api("pproject_dependendent_repositories", manager, package)

    def project_contributors(self, manager: str, package: str) -> Any:
        """
        Get users that have contributed to a given project.

        Args:
            manager: package manager
            package: package name
        Returns:
            List of dicts of project contributor info from libraries.io.
        """

        return search_api("pproject_contributors", manager, package)

    def project_sourcerank(self, manager: str, package: str) -> Any:
        """
        Get breakdown of SourceRank score for a given project.

        Args:
            manager: package manager
            package: package name
        Returns:
            Dict of sourcerank info response from libraries.io.
        """

        return search_api("pproject_sourcerank", manager, package)

    def project_usage(selfself, manager: str, package: str) -> Any:
        """
        Get breakdown of usage for a given project.

        Args:
            manager: package manager
            package: package name
        Returns:
            Dict with info about usage from libraries.io.
        """

        return search_api("pproject_usage", manager, package)

    def project_search(self, **kwargs):
        """
        Search for projects. Accepts keyword arguments only.

        Args:
            filters (dict): optional dict of form
                dict(languages="python", keywords="data", 
                licenses="my_license", manager="pypi")

            sort (str): (optional) one of rank, stars, 
                dependents_count, dependent_repos_count, 
                latest_release_published_at, contributions_count, created_at
        
        Returns:
            List of dicts of project info from libraries.io.
        """

        return search_api("special_project_search", **kwargs)

    def repository(self, host: (str), owner: (str), repo: (str)) -> Any:
        """
        Return information about a repository and its versions.

        Args:
            host: host host name (e.g. GitHub)
            owner: owner
            repo: repo
        Returns:
            List of dicts of info about a repository from libraries.io.
        """

        return search_api("repository", host, owner, repo)

    def repository_dependencies(self, host: (str), owner: (str), repo: (str)) -> Any:
        """
        Return information about a repository's dependencies.

        Args:
            host: host provider name (e.g. GitHub)
            owner: owner
            repo: repo
        Returns:
            Dict of repo dependency info from libraries.io.
        """

        return search_api("repository_dependencies", host, owner, repo)

    def repository_projects(self, host: (str), owner: (str), repo: (str)) -> Any:
        """
        Get a list of packages referencing the given repository.

        Args:
            host: host provider name (e.g. GitHub)
            owner: owner
            repo: repo
        Returns:
            List of dicts of packages referencing a repo from libraries.io.
        """

        return search_api("repository_projects", host, owner, repo)

    def user(self, host: (str), user: (str)) -> Any:
        """
        Return information about a user.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
        Dict of info about user from libraries.io.
        """
        return search_api("user", host, user)

    def user_repositories(self, host: (str), user: (str)) -> Any:
        """
        Return information about a user's repos.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
            List of dicts with info about user repos from libraries.io.
        """
        return search_api("user_repositories", host, user)

    def user_packages(self, host: (str), user: (str)) -> Any:
        """
        Return information about packages using a user's repos.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
            List of dicts of package info from libraries.io.
        """
        return search_api("user_packages", host, user)

    def user_packages_contributions(self, host: (str), user: (str)) -> Any:
        """
        Return information about packages a user has contributed to.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
            List of dicts with user package contribution info from libraries.io.
        """
        return search_api("user_packages_contributions", host, user)

    def user_repository_contributions(self, host: (str), user: (str)) -> Any:
        """
        Return information about repositories a user has contributed to.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
            (list): list of dicts response from libraries.io
        """
        return search_api("user_repositories_contributions", host, user)

    def user_dependencies(self, host, user):
        """
        Return a list of unique user's repositories' dependencies.

        Ordered by frequency of use in those repositories.

        Args:
            host: host (e.g. github)
            user: username
        Returns:
            Liist of dicts with user package dependency info.
        """
        return search_api("user_dependencies", host, user)


# From the command line you can call any public function by name with arguments
if __name__ == "__main__":
    fire.Fire(Search)

    # manually testing actions
    # api = Search()

    # t = api.user("github", "discdiver")
    # print(t)

    # x = set_pages(1, 3)
    # print(x)

    # d = api.project_search(
    #    sort="latest_release_published_at",
    #    filters=dict(keywords="visualization", manager="pypi"),
    # )
    # print(d)



from setuptools import setup, find_packages


def readme():
    with open("docs/README.rst") as f:
        return f.read()


setup_requirements = ["pytest-runner"]

test_requirements = ["pytest"]

requirements = ["requests>=2", "fire>0.1.0"]

setup(
    name="pybraries",
    version="0.1.0",
    author="Jeff Hale",
    author_email="jeffmshale@gmail.com",
    description="A Python wrapper for the libraries.io API",
    long_description=readme(),
    url="https://github.com/pybraries/pybraries/",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: BSD License",
    ],
    include_package_data=True,
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    zip_safe=False,
)



# subscription_helpers.py
from requests.exceptions import HTTPError
import fire
from pybraries.helpers import sess, clear_params
from pybraries.make_request import make_request


def sub_api(action, *args, **kwargs):
    url_end_list = [
        "https://libraries.io/api/subscriptions"
    ]  # start of list to build url
    more_args = []  # for unpacking args
    url_combined = ""  # final string url
    call_type = ""  # post, put or delete
    manager = ""
    package = ""
    kind = "get"

    if kwargs:
        if "manager" in kwargs:
            manager = kwargs["manager"]
        if "package" in kwargs:
            package = kwargs["package"]

    if args:
        if args[0]:
            manager = args[0]
        if args[1]:
            package = args[1]

    if action == "list_subscribed":
        url_combined = "/".join(url_end_list)
        resp = make_request(url_combined, kind)
        return resp

    if action == "check_subscribed":
        # do we need to check for args?
        url_end_list.append(manager)
        url_end_list.append(package)
        url_combined = "/".join(url_end_list)

        check = make_request(url_combined, kind)

        if check is not None:
            r_bool = True
        else:
            r_bool = False

        return r_bool

    if action == "subscribe":
        kind = "post"
        url_end_list.append(manager)
        url_end_list.append(package)

        url_combined = "/".join(url_end_list)
        return make_request(url_combined, kind)

    if action == "update_subscribe":
        kind = "put"
        # not implemented - seems libraries.io api has bug
        # if implemented in future, adjust modules in readme
        url_end_list.append(manager)
        url_end_list.append(package)

        url_combined = "/".join(url_end_list)
        return make_request(url_combined, kind)

    if action == "delete_subscribe":
        kind = "delete"
        url_end_list.append(manager)
        url_end_list.append(package)
        url_combined = "/".join(url_end_list)

        # first check if subscribed. Must be done before build url.
        check_pkg_subscribed = sub_api(
            "check_subscribed", manager=manager, package=package
        )

        if check_pkg_subscribed is False:
            msg = f"Unsubscribe unnecessary. You are not subscribed to {package}."
            return msg
        else:
            return make_request(url_combined, kind)

        # prerelease option ccurrently not changeable for subscribe or update
        # (perhaps due to libraries.io api glitch)
        def __check_prerelease(*args, **kwargs):
            prerelease = kwargs.pop("include_prerelease", "")
            return prerelease



# set_pages.py
from requests.exceptions import HTTPError
from pybraries.helpers import sess


def set_pages(per_page=30, page=1):
    """
    Change pagination settings.

    Args:
        per_page (int): default=30 number of items per page.  max=100
        page (int): default=1 package name

    Returns:
        response (str): message with pagination information
    """

    if not all(isinstance(i, int) for i in [page, per_page]):
        raise TypeError("Must be an integer")
    if page < 1:
        raise ValueError("page must be an integer > 1")
    if per_page > 100 or per_page < 1:
        raise ValueError("perpage must be an integer between 1 and 100, inclusive")

    sess.params["page"] = page  # 1 returns page 1 of results, 2 returns page 2, etc.
    sess.params["per_page"] = per_page  # 30 is libraries api default, max is 100

    return f"per_page set to {per_page} and page set to {page}."
